// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef MPVQT_VERSION_H
#define MPVQT_VERSION_H

#define MPVQT_VERSION_STRING "1.2.0"
#define MPVQT_VERSION_MAJOR 1
#define MPVQT_VERSION_MINOR 2
#define MPVQT_VERSION_PATCH 0
#define MPVQT_VERSION ((1<<16)|(2<<8)|(0))

#endif
